@@ -166,9 +166,10 @@ static int ratbagd_get_devices(sd_bus *bus,
 
 static const sd_bus_vtable ratbagd_vtable[] = {
 	SD_BUS_VTABLE_START(0),
+	SD_BUS_PROPERTY("APIVersion", "i", 0, offsetof(struct ratbagd, api_version), SD_BUS_VTABLE_PROPERTY_CONST),
 	SD_BUS_PROPERTY("Devices", "ao", ratbagd_get_devices, 0, SD_BUS_VTABLE_PROPERTY_EMITS_CHANGE),
 #ifdef RATBAG_DEVELOPER_EDITION
-	SD_BUS_METHOD("ResetTestDevice", "", "u", ratbagd_reset_test_device, SD_BUS_VTABLE_UNPRIVILEGED),
+	SD_BUS_METHOD("LoadTestDevice", "s", "i", ratbagd_load_test_device, SD_BUS_VTABLE_UNPRIVILEGED),
 #endif /* RATBAG_DEVELOPER_EDITION */
 	SD_BUS_VTABLE_END,
 };
@@ -200,7 +201,7 @@ static void ratbagd_process_device(struct ratbagd *ctx,
 		/* device was removed, unlink it and destroy our context */
 		if (device) {
 			ratbagd_device_unlink(device);
-			ratbagd_device_free(device);
+			ratbagd_device_unref(device);
 
 			(void) sd_bus_emit_properties_changed(ctx->bus,
 							      RATBAGD_OBJ_ROOT,
@@ -283,7 +284,7 @@ static struct ratbagd *ratbagd_free(struct ratbagd *ctx)
 
 	RATBAGD_DEVICE_FOREACH_SAFE(device, tmp, ctx) {
 		ratbagd_device_unlink(device);
-		ratbagd_device_free(device);
+		ratbagd_device_unref(device);
 	}
 
 	ctx->bus = sd_bus_flush_close_unref(ctx->bus);
@@ -345,6 +346,7 @@ static int ratbagd_new(struct ratbagd **out)
 	int r;
 
 	ctx = zalloc(sizeof(*ctx));
+	ctx->api_version = RATBAGD_API_VERSION;
 
 	r = sd_event_default(&ctx->event);
 	if (r < 0)
@@ -521,6 +523,83 @@ static int ratbagd_run(struct ratbagd *ctx)
 	return sd_event_loop(ctx->event);
 }
 
+static bool install_ratbagd_devel_dbus_policy(void)
+{
+	bool rc = true;
+#ifdef RATBAG_DEVELOPER_EDITION
+	_cleanup_close_ int in = -1, out = -1;
+	int nread, nwrite;
+	char buf[40960];
+	int r;
+	sd_bus_error error = SD_BUS_ERROR_NULL;
+	sd_bus_message *m = NULL;
+	sd_bus *bus = NULL;
+
+	rc = false;
+
+	log_verbose("Installing DBus policy file to %s\n", DBUS_POLICY_DST);
+
+	in = open(DBUS_POLICY_SRC, O_RDONLY);
+	if (in == -1) {
+		log_error("Failed to source policy file: %m\n");
+		goto out;
+	}
+	out = open(DBUS_POLICY_DST, O_CREAT|O_WRONLY, 0644);
+	if (out == -1) {
+		log_error("Failed to open destination: %m\n");
+		goto out;
+	}
+
+
+	nread = read(in, buf, sizeof(buf));
+	if (nread < 0) {
+		log_error("Failed to read policy file: %m\n");
+		goto out;
+	}
+
+	nwrite = write(out, buf, nread);
+	if (nread != nwrite) {
+		log_error("Failed to write policy file: %m\n");
+		goto out;
+	}
+
+	/* Now poke DBus to reload itself */
+	r = sd_bus_open_system(&bus);
+	if (r < 0) {
+		log_error("Unable to open system bus: %s\n", strerror(-r));
+		goto out;
+	}
+	r = sd_bus_call_method(bus, "org.freedesktop.DBus",
+			       "/org/freedesktop/DBus",
+			       "org.freedesktop.DBus",
+			       "ReloadConfig",
+			       &error,
+			       &m,
+			       "");
+	if (r < 0) {
+		log_error("Failed to call DBus ReloadConfig: %s\n",
+			  error.message);
+		goto out;
+	}
+
+	rc = true;
+
+out:
+	sd_bus_error_free(&error);
+	sd_bus_message_unref(m);
+	sd_bus_unref(bus);
+
+#endif
+	return rc;
+}
+
+static void remove_ratbagd_devel_dbus_policy(void)
+{
+#ifdef RATBAG_DEVELOPER_EDITION
+	unlink(DBUS_POLICY_DST);
+#endif
+}
+
 int main(int argc, char *argv[])
 {
 	struct ratbagd *ctx = NULL;
@@ -550,6 +629,9 @@ int main(int argc, char *argv[])
 		}
 	}
 
+	if (!install_ratbagd_devel_dbus_policy())
+		goto exit;
+
 	r = ratbagd_new(&ctx);
 	if (r < 0)
 		goto exit;
@@ -558,6 +640,7 @@ int main(int argc, char *argv[])
 
 	r = ratbagd_run(ctx);
 
+	remove_ratbagd_devel_dbus_policy();
 exit:
 	ratbagd_free(ctx);
 
