@@ -56,6 +56,10 @@
 #define HIDPP_CAP_ONBOARD_PROFILES_8100			(1 << 6)
 #define HIDPP_CAP_LED_SW_CONTROL_1300			(1 << 7)
 #define HIDPP_CAP_ADJUSTIBLE_REPORT_RATE_8060		(1 << 8)
+#define HIDPP_CAP_BATTERY_VOLTAGE_1001			(1 << 9)
+#define HIDPP_CAP_RGB_EFFECTS_8071			(1 << 10)
+
+#define HIDPP_HIDDEN_FEATURE				(1 << 6)
 
 struct hidpp20drv_data {
 	struct hidpp20_device *dev;
@@ -335,7 +339,7 @@ hidpp20drv_read_led_8070(struct ratbag_led *led, struct hidpp20drv_data* drv_dat
 	struct hidpp20_color_led_info info;
 	int rc;
 
-	led_info = &drv_data->led_infos.color_leds[led->index];
+	led_info = &drv_data->led_infos.color_leds_8070[led->index];
 	profile = &drv_data->profiles->profiles[led->profile->index];
 	h_led = &profile->leds[led->index];
 
@@ -356,7 +360,7 @@ hidpp20drv_read_led_8070(struct ratbag_led *led, struct hidpp20drv_data* drv_dat
 
 	/* pre-filled, only override if unknown */
 	if (led->type == RATBAG_LED_TYPE_UNKNOWN)
-		led->type = hidpp20_8070_get_location_mapping(led_info->location);
+		led->type = hidpp20_led_get_location_mapping(led_info->location);
 	led->color.red = h_led->color.red;
 	led->color.green = h_led->color.green;
 	led->color.blue = h_led->color.blue;
@@ -402,6 +406,81 @@ hidpp20drv_read_led_8070(struct ratbag_led *led, struct hidpp20drv_data* drv_dat
 	}
 }
 
+static void
+hidpp20drv_read_led_8071(struct ratbag_led *led, struct hidpp20drv_data* drv_data)
+{
+	struct hidpp20_profile *profile;
+	struct hidpp20_led *h_led;
+	struct hidpp20_rgb_device_info device_info;
+	struct hidpp20_rgb_cluster_info *cluster_info;
+	int rc;
+
+	hidpp20_rgb_effects_get_device_info(drv_data->dev, &device_info);
+	cluster_info = &drv_data->led_infos.color_leds_8071[led->index];
+	profile = &drv_data->profiles->profiles[led->profile->index];
+	h_led = &profile->leds[led->index];
+
+	switch (h_led->mode) {
+	case HIDPP20_LED_ON:
+		led->mode = RATBAG_LED_ON;
+		break;
+	case HIDPP20_LED_CYCLE:
+		led->mode = RATBAG_LED_CYCLE;
+		break;
+	case HIDPP20_LED_BREATHING:
+		led->mode = RATBAG_LED_BREATHING;
+		break;
+	default:
+		led->mode = RATBAG_LED_OFF;
+		break;
+	}
+
+	/* pre-filled, only override if unknown */
+	if (led->type == RATBAG_LED_TYPE_UNKNOWN)
+		led->type = hidpp20_led_get_location_mapping(cluster_info->location);
+	led->color.red = h_led->color.red;
+	led->color.green = h_led->color.green;
+	led->color.blue = h_led->color.blue;
+	led->ms = h_led->period;
+	led->brightness = h_led->brightness * 255 / 100;
+
+	if (device_info.ext_caps & HIDPP20_COLOR_LED_INFO_EXT_CAP_MONOCHROME_ONLY)
+		led->colordepth = RATBAG_LED_COLORDEPTH_MONOCHROME;
+	else
+		led->colordepth = RATBAG_LED_COLORDEPTH_RGB_888;
+
+	for (int i = 0; i < cluster_info->num_effects; i++) {
+		struct hidpp20_rgb_effect_info ei;
+		rc = hidpp20_rgb_effects_get_effect_info(drv_data->dev,
+							 cluster_info->index,
+							 i, &ei);
+		if (rc < 0)
+			break;
+
+		switch (ei.effect_id) {
+		case HIDPP20_COLOR_LED_ZONE_EFFECT_DISABLED:
+			ratbag_led_set_mode_capability(led, RATBAG_LED_OFF);
+			break;
+		case HIDPP20_COLOR_LED_ZONE_EFFECT_FIXED:
+			ratbag_led_set_mode_capability(led, RATBAG_LED_ON);
+			break;
+		case HIDPP20_COLOR_LED_ZONE_EFFECT_CYCLING:
+			ratbag_led_set_mode_capability(led, RATBAG_LED_CYCLE);
+			break;
+		case HIDPP20_COLOR_LED_ZONE_EFFECT_WAVE:
+		case HIDPP20_COLOR_LED_ZONE_EFFECT_BREATHING:
+			ratbag_led_set_mode_capability(led, RATBAG_LED_BREATHING);
+			break;
+		default:
+			log_bug_libratbag(led->profile->device->ratbag,
+					  "%s: Unknown effect id %d\n",
+					  led->profile->device->name,
+					  ei.effect_id);
+			break;
+		}
+	}
+}
+
 static void
 hidpp20drv_read_led(struct ratbag_led *led)
 {
@@ -410,6 +489,8 @@ hidpp20drv_read_led(struct ratbag_led *led)
 
 	if (drv_data->capabilities & HIDPP_CAP_COLOR_LED_EFFECTS_8070)
 		hidpp20drv_read_led_8070(led, drv_data);
+	else if (drv_data->capabilities & HIDPP_CAP_RGB_EFFECTS_8071)
+		hidpp20drv_read_led_8071(led, drv_data);
 	else if (drv_data->capabilities & HIDPP_CAP_LED_SW_CONTROL_1300)
 		hidpp20drv_read_led_1300(led, drv_data);
 }
@@ -606,8 +687,8 @@ hidpp20drv_update_led_1300(struct ratbag_led *led, struct hidpp20drv_data *data)
 }
 
 static int
-hidpp20drv_update_led_8070(struct ratbag_led *led, struct ratbag_profile* profile,
-						   struct hidpp20drv_data *drv_data)
+hidpp20drv_update_led_8070_8071(struct ratbag_led *led, struct ratbag_profile* profile,
+				struct hidpp20drv_data *drv_data)
 {
 	struct hidpp20_profile *h_profile;
 	struct hidpp20_led *h_led;
@@ -649,8 +730,9 @@ hidpp20drv_update_led(struct ratbag_led *led)
 	struct ratbag_device *device = profile->device;
 	struct hidpp20drv_data *drv_data = ratbag_get_drv_data(device);
 
-	if (drv_data->capabilities & HIDPP_CAP_COLOR_LED_EFFECTS_8070)
-		return hidpp20drv_update_led_8070(led, profile, drv_data);
+	if ((drv_data->capabilities & HIDPP_CAP_COLOR_LED_EFFECTS_8070) |
+	    (drv_data->capabilities & HIDPP_CAP_RGB_EFFECTS_8071))
+		return hidpp20drv_update_led_8070_8071(led, profile, drv_data);
 
 	if (drv_data->capabilities & HIDPP_CAP_LED_SW_CONTROL_1300)
 		return hidpp20drv_update_led_1300(led, drv_data);
@@ -897,6 +979,34 @@ out:
 	return rc;
 }
 
+static int
+hidpp20drv_update_report_rate_8100(struct ratbag_profile *profile, int hz)
+{
+	struct ratbag_device *device = profile->device;
+	struct hidpp20drv_data *drv_data = ratbag_get_drv_data(device);
+	struct hidpp20_profile *h_profile;
+
+	h_profile = &drv_data->profiles->profiles[profile->index];
+	h_profile->report_rate = hz;
+
+	return RATBAG_SUCCESS;
+}
+
+static int
+hidpp20drv_update_report_rate(struct ratbag_profile *profile, int hz)
+{
+	struct ratbag_device *device = profile->device;
+	struct hidpp20drv_data *drv_data = ratbag_get_drv_data(device);
+
+	if (drv_data->capabilities & HIDPP_CAP_ONBOARD_PROFILES_8100)
+		return hidpp20drv_update_report_rate_8100(profile, hz);
+
+	if (drv_data->capabilities & HIDPP_CAP_ADJUSTIBLE_REPORT_RATE_8060)
+		return -ENOTSUP;
+
+	return -ENOTSUP;
+}
+
 static int
 hidpp20drv_read_special_key_mouse(struct ratbag_device *device)
 {
@@ -940,19 +1050,15 @@ hidpp20drv_read_kbd_reprogrammable_key(struct ratbag_device *device)
 }
 
 static int
-hidpp20drv_read_color_leds(struct ratbag_device *device)
+hidpp20drv_read_color_leds_8070(struct hidpp20drv_data *drv_data)
 {
-	struct hidpp20drv_data *drv_data = ratbag_get_drv_data(device);
 	int rc;
 
-	if (!(drv_data->capabilities & HIDPP_CAP_COLOR_LED_EFFECTS_8070))
-		return 0;
-
-	free(drv_data->led_infos.color_leds);
-	drv_data->led_infos.color_leds = NULL;
+	free(drv_data->led_infos.color_leds_8070);
+	drv_data->led_infos.color_leds_8070 = NULL;
 	drv_data->num_leds = 0;
 
-	rc = hidpp20_color_led_effects_get_zone_infos(drv_data->dev, &drv_data->led_infos.color_leds);
+	rc = hidpp20_color_led_effects_get_zone_infos(drv_data->dev, &drv_data->led_infos.color_leds_8070);
 
 	if (rc > 0) {
 		drv_data->num_leds = rc;
@@ -962,6 +1068,38 @@ hidpp20drv_read_color_leds(struct ratbag_device *device)
 	return rc;
 }
 
+static int
+hidpp20drv_read_color_leds_8071(struct hidpp20drv_data *drv_data)
+{
+	int rc;
+
+	free(drv_data->led_infos.color_leds_8071);
+	drv_data->led_infos.color_leds_8071 = NULL;
+	drv_data->num_leds = 0;
+
+	rc = hidpp20_rgb_effects_get_cluster_infos(drv_data->dev, &drv_data->led_infos.color_leds_8071);
+
+	if (rc > 0) {
+		drv_data->num_leds = rc;
+		rc = 0;
+	}
+
+	return rc;
+}
+
+static int
+hidpp20drv_read_color_leds(struct ratbag_device *device)
+{
+	struct hidpp20drv_data *drv_data = ratbag_get_drv_data(device);
+
+	if(drv_data->capabilities & HIDPP_CAP_COLOR_LED_EFFECTS_8070)
+		return hidpp20drv_read_color_leds_8070(drv_data);
+	else if (drv_data->capabilities & HIDPP_CAP_RGB_EFFECTS_8071)
+		return hidpp20drv_read_color_leds_8071(drv_data);
+
+	return 0;
+}
+
 static int
 hidpp20drv_read_leds(struct ratbag_device *device)
 {
@@ -1078,6 +1216,10 @@ hidpp20drv_init_feature(struct ratbag_device *device, uint16_t feature)
 	if (rc < 0)
 		return rc;
 
+	/* check if it is a hidden feature */
+	if (feature_type & HIDPP_HIDDEN_FEATURE)
+		return 0;
+
 	switch (feature) {
 	case HIDPP_PAGE_ROOT:
 	case HIDPP_PAGE_FEATURE_SET:
@@ -1121,6 +1263,22 @@ hidpp20drv_init_feature(struct ratbag_device *device, uint16_t feature)
 		drv_data->capabilities |= HIDPP_CAP_BATTERY_LEVEL_1000;
 		break;
 	}
+	case HIDPP_PAGE_BATTERY_VOLTAGE: {
+		uint16_t voltage;
+		enum hidpp20_battery_voltage_status status;
+
+		rc = hidpp20_batteryvoltage_get_battery_voltage(drv_data->dev, &voltage);
+		if (rc < 0)
+			return rc;
+
+		status = rc;
+
+		log_debug(ratbag, "device battery voltage is %dmv, status %02x \n",
+			  voltage, status);
+
+		drv_data->capabilities |= HIDPP_CAP_BATTERY_VOLTAGE_1001;
+		break;
+	}
 	case HIDPP_PAGE_KBD_REPROGRAMMABLE_KEYS: {
 		log_debug(ratbag, "device has programmable keys/buttons\n");
 		drv_data->capabilities |= HIDPP_CAP_KBD_REPROGRAMMABLE_KEYS_1b00;
@@ -1155,6 +1313,18 @@ hidpp20drv_init_feature(struct ratbag_device *device, uint16_t feature)
 		device->num_leds = drv_data->num_leds;
 		break;
 	}
+	case HIDPP_PAGE_RGB_EFFECTS: {
+		log_debug(ratbag, "device has color effects\n");
+		drv_data->capabilities |= HIDPP_CAP_RGB_EFFECTS_8071;
+
+		/* we read the profile once to get the correct number of
+		 * supported leds. */
+		if (hidpp20drv_read_color_leds(device))
+			return 0;
+
+		device->num_leds = drv_data->num_leds;
+		break;
+	}
 	case HIDPP_PAGE_LED_SW_CONTROL: {
 		log_debug(ratbag, "device has non-rgb leds\n");
 		drv_data->capabilities |= HIDPP_CAP_LED_SW_CONTROL_1300;
@@ -1213,9 +1383,15 @@ hidpp20drv_commit(struct ratbag_device *device)
 		if (!profile->dirty)
 			continue;
 
+		rc = hidpp20drv_update_report_rate(profile, profile->hz);
+		if (rc)
+			return RATBAG_ERROR_DEVICE;
+
 		ratbag_profile_for_each_resolution(profile, resolution) {
-			if (resolution->is_active)
+			if (resolution->is_active) {
+				log_raw(device->ratbag, "dpi index: %d, profile %d\n", resolution->index, profile->index);
 				dpi_index = resolution->index;
+			}
 
 			rc = hidpp20drv_update_resolution_dpi(resolution,
 							      resolution->dpi_x,
@@ -1254,9 +1430,14 @@ hidpp20drv_commit(struct ratbag_device *device)
 			return RATBAG_ERROR_DEVICE;
 
 		list_for_each(profile, &device->profiles, link) {
-			if (profile->is_active)
+			if (profile->is_active) {
+				ratbag_profile_for_each_resolution(profile, resolution) {
+					if (resolution->is_active)
+						dpi_index = resolution->index;
+				}
 				hidpp20_onboard_profiles_set_current_dpi_index(drv_data->dev,
 									       dpi_index);
+			}
 		}
 	}
 
@@ -1318,7 +1499,8 @@ hidpp20drv_remove(struct ratbag_device *device)
 
 	if (drv_data->profiles)
 		hidpp20_onboard_profiles_destroy(drv_data->profiles);
-	free(drv_data->led_infos.color_leds);
+	free(drv_data->led_infos.color_leds_8070);
+	free(drv_data->led_infos.color_leds_8071);
 	free(drv_data->controls);
 	free(drv_data->sensors);
 	if (drv_data->dev)
