@@ -32,6 +32,8 @@
 #include <stdarg.h>
 #include <stddef.h>
 
+#include "libratbag-util.h"
+
 #define HIDPP_RECEIVER_IDX			0xFF
 #define HIDPP_WIRED_DEVICE_IDX			0x00
 
@@ -118,10 +120,10 @@ enum ratbag_button_type
 hidpp20_1b04_get_physical_mapping(uint16_t value);
 
 enum ratbag_led_type
-hidpp20_8070_get_location_mapping(uint16_t value);
+hidpp20_led_get_location_mapping(uint16_t value);
 
 const char *
-hidpp20_8070_get_location_mapping_name(uint16_t value);
+hidpp20_led_get_location_mapping_name(uint16_t value);
 
 const char *
 hidpp20_1b04_get_logical_mapping_name(uint16_t value);
@@ -145,6 +147,9 @@ hidpp_log(struct hidpp_device *dev,
 	  ...)
 	__attribute__ ((format (printf, 3, 4)));
 
+char *
+hidpp_buffer_to_string(const uint8_t *buf, size_t len);
+
 void
 hidpp_log_buffer(struct hidpp_device *dev,
 		 enum hidpp_log_priority priority,
@@ -163,62 +168,30 @@ hidpp_log_buffer(struct hidpp_device *dev,
 
 uint16_t hidpp_crc_ccitt(uint8_t *data, unsigned int length);
 
-static inline uint16_t
-hidpp_get_unaligned_be_u16(uint8_t *buf)
-{
-	return (buf[0] << 8) | buf[1];
-}
-
 static inline uint16_t
 hidpp_be_u16_to_cpu(uint16_t data)
 {
-	return hidpp_get_unaligned_be_u16((uint8_t *)&data);
-}
-
-static inline void
-hidpp_set_unaligned_be_u16(uint8_t *buf, uint16_t value)
-{
-	buf[0] = value >> 8;
-	buf[1] = value & 0xFF;
+	return get_unaligned_be_u16((uint8_t *)&data);
 }
 
 static inline uint16_t
 hidpp_cpu_to_be_u16(uint16_t data)
 {
 	uint16_t result;
-	hidpp_set_unaligned_be_u16((uint8_t *)&result, data);
+	set_unaligned_be_u16((uint8_t *)&result, data);
 	return result;
 }
 
-static inline uint16_t
-hidpp_get_unaligned_le_u16(uint8_t *buf)
-{
-	return (buf[1] << 8) | buf[0];
-}
-
 static inline uint16_t
 hidpp_le_u16_to_cpu(uint16_t data)
 {
-	return hidpp_get_unaligned_le_u16((uint8_t *)&data);
-}
-
-static inline void
-hidpp_set_unaligned_le_u16(uint8_t *buf, uint16_t value)
-{
-	buf[0] = value & 0xFF;
-	buf[1] = value >> 8;
+	return get_unaligned_le_u16((uint8_t *)&data);
 }
 
 static inline uint16_t
 hidpp_cpu_to_le_u16(uint16_t data)
 {
 	uint16_t result;
-	hidpp_set_unaligned_le_u16((uint8_t *)&result, data);
+	set_unaligned_le_u16((uint8_t *)&result, data);
 	return result;
 }
-
-static inline uint32_t
-hidpp_get_unaligned_be_u32(uint8_t *buf)
-{
-	return (buf[0] << 24) | (buf[1] << 16) | (buf[2] << 8) | buf[3];
-}
