@@ -161,7 +161,6 @@ hidpp10_build_dpi_table_from_dpi_info(struct hidpp10_device *dev,
 	return 0;
 }
 
-
 static int
 hidpp10_request_command(struct hidpp10_device *dev, union hidpp10_message *msg)
 {
@@ -171,6 +170,7 @@ hidpp10_request_command(struct hidpp10_device *dev, union hidpp10_message *msg)
 	int ret;
 	uint8_t hidpp_err = 0;
 	int command_size;
+	_cleanup_free_ char *rxdata = NULL, *txdata = NULL;
 
 	switch (msg->msg.report_id) {
 	case REPORT_ID_SHORT:
@@ -186,9 +186,19 @@ hidpp10_request_command(struct hidpp10_device *dev, union hidpp10_message *msg)
 	/* create the expected header */
 	expected_header = *msg;
 
+	txdata = hidpp_buffer_to_string(&msg->data[4], command_size - 4);
+	hidpp_log_raw(&dev->base, "hidpp10 tx:  %02x | %02x | %02x | %02x | %s\n",
+		      msg->msg.report_id,
+		      msg->msg.device_idx,
+		      msg->msg.sub_id,
+		      msg->msg.address,
+		      txdata);
+
 	/* response message length doesn't depend on request length */
+#if 0
 	hidpp_log_buf_raw(&dev->base, "  expected_header:		?? ", &expected_header.data[1], 3);
 	hidpp_log_buf_raw(&dev->base, "  expected_error_dev:	", expected_error_dev.data, SHORT_MESSAGE_LENGTH);
+#endif
 
 	/* Send the message to the Device */
 	ret = hidpp_write_command(&dev->base, msg->data, command_size);
@@ -238,8 +248,15 @@ hidpp10_request_command(struct hidpp10_device *dev, union hidpp10_message *msg)
 		goto out_err;
 	}
 
+	rxdata = hidpp_buffer_to_string(&read_buffer.data[4], ret - 4);
+	hidpp_log_raw(&dev->base, "hidpp10 rx:  %02x | %02x | %02x | %02x | %s\n",
+		      read_buffer.msg.report_id,
+		      read_buffer.msg.device_idx,
+		      read_buffer.msg.sub_id,
+		      read_buffer.msg.address,
+		      rxdata);
+
 	if (!hidpp_err) {
-		hidpp_log_buf_raw(&dev->base, "  received: ", read_buffer.data, ret);
 		/* copy the answer for the caller */
 		*msg = read_buffer;
 	}
@@ -277,7 +294,8 @@ hidpp10_get_hidpp_notifications(struct hidpp10_device *dev,
 	union hidpp10_message notifications = CMD_HIDPP_NOTIFICATIONS(idx, GET_REGISTER_REQ);
 	int res;
 
-	hidpp_log_raw(&dev->base, "Fetching HID++ notifications\n");
+	hidpp_log_raw(&dev->base, "Fetching HID++ notifications (%#02x)\n",
+		      __CMD_HIDPP_NOTIFICATIONS);
 
 	res = hidpp10_request_command(dev, &notifications);
 	if (res)
@@ -298,7 +316,8 @@ hidpp10_set_hidpp_notifications(struct hidpp10_device *dev,
 	union hidpp10_message notifications = CMD_HIDPP_NOTIFICATIONS(idx, SET_REGISTER_REQ);
 	int res;
 
-	hidpp_log_raw(&dev->base, "Setting HID++ notifications\n");
+	hidpp_log_raw(&dev->base, "Setting HID++ notifications (%#02x\n",
+		      __CMD_HIDPP_NOTIFICATIONS);
 
 	notifications.msg.parameters[0] = reporting_flags & 0xFF;
 	notifications.msg.parameters[1] = (reporting_flags >> 8) & 0x1F;
@@ -333,7 +352,8 @@ hidpp10_get_individual_features(struct hidpp10_device *dev,
 	union hidpp10_message features = CMD_ENABLE_INDIVIDUAL_FEATURES(idx, GET_REGISTER_REQ);
 	int res;
 
-	hidpp_log_raw(&dev->base, "Fetching individual features\n");
+	hidpp_log_raw(&dev->base, "Fetching individual features (%#02x)\n",
+		      __CMD_ENABLE_INDIVIDUAL_FEATURES);
 
 	res = hidpp10_request_command(dev, &features);
 	if (res)
@@ -356,7 +376,8 @@ hidpp10_set_individual_features(struct hidpp10_device *dev,
 	union hidpp10_message mode = CMD_ENABLE_INDIVIDUAL_FEATURES(idx, SET_REGISTER_REQ);
 	int res;
 
-	hidpp_log_raw(&dev->base, "Setting individual features\n");
+	hidpp_log_raw(&dev->base, "Setting individual features (%#02x)\n",
+		      __CMD_ENABLE_INDIVIDUAL_FEATURES);
 
 	mode.msg.device_idx = dev->index;
 
@@ -700,7 +721,7 @@ hidpp10_write_profile_directory(struct hidpp10_device *dev)
 	}
 
 	crc = hidpp_crc_ccitt(bytes, HIDPP10_PAGE_SIZE - 2);
-	hidpp_set_unaligned_be_u16(&bytes[HIDPP10_PAGE_SIZE - 2], crc);
+	set_unaligned_be_u16(&bytes[HIDPP10_PAGE_SIZE - 2], crc);
 
 	res = hidpp10_send_hot_payload(dev,
 				       0x00, 0x0000, /* destination: RAM */
@@ -782,7 +803,8 @@ hidpp10_get_current_profile(struct hidpp10_device *dev, uint8_t *current_profile
 	unsigned i;
 	uint8_t type, page, offset;
 
-	hidpp_log_raw(&dev->base, "Fetching current profile\n");
+	hidpp_log_raw(&dev->base, "Fetching current profile (%#02x)\n",
+		      __CMD_PROFILE);
 
 	res = hidpp10_request_command(dev, &profile);
 	if (res)
@@ -829,7 +851,8 @@ hidpp10_set_internal_current_profile(struct hidpp10_device *dev,
 	union hidpp10_message profile = CMD_PROFILE(idx, SET_REGISTER_REQ);
 	int8_t page, offset;
 
-	hidpp_log_raw(&dev->base, "Setting current profile\n");
+	hidpp_log_raw(&dev->base, "Setting current profile (%#02x)\n",
+		      __CMD_PROFILE);
 
 	profile.msg.parameters[0] = profile_type;
 
@@ -964,9 +987,9 @@ hidpp10_fill_dpi_modes_16(struct hidpp10_device *dev,
 		struct _hidpp10_dpi_mode_16 *dpi = &dpi_list[i];
 
 		be = (uint8_t*)&dpi->xres;
-		profile->dpi_modes[i].xres = hidpp10_get_dpi_value(dev, hidpp_get_unaligned_be_u16(be));
+		profile->dpi_modes[i].xres = hidpp10_get_dpi_value(dev, get_unaligned_be_u16(be));
 		be = (uint8_t*)&dpi->yres;
-		profile->dpi_modes[i].yres = hidpp10_get_dpi_value(dev, hidpp_get_unaligned_be_u16(be));
+		profile->dpi_modes[i].yres = hidpp10_get_dpi_value(dev, get_unaligned_be_u16(be));
 
 		profile->dpi_modes[i].led[0] = dpi->led1 == 0x2;
 		profile->dpi_modes[i].led[1] = dpi->led2 == 0x2;
@@ -988,9 +1011,9 @@ hidpp10_write_dpi_modes_16(struct hidpp10_device *dev,
 		struct _hidpp10_dpi_mode_16 *dpi = &dpi_list[i];
 
 		be = (uint8_t*)&dpi->xres;
-		hidpp_set_unaligned_be_u16(be, hidpp10_get_dpi_mapping(dev, profile->dpi_modes[i].xres));
+		set_unaligned_be_u16(be, hidpp10_get_dpi_mapping(dev, profile->dpi_modes[i].xres));
 		be = (uint8_t*)&dpi->yres;
-		hidpp_set_unaligned_be_u16(be, hidpp10_get_dpi_mapping(dev, profile->dpi_modes[i].yres));
+		set_unaligned_be_u16(be, hidpp10_get_dpi_mapping(dev, profile->dpi_modes[i].yres));
 
 		dpi->led1 = profile->dpi_modes[i].led[0] ? 0x02 : 0x01;
 		dpi->led2 = profile->dpi_modes[i].led[1] ? 0x02 : 0x01;
@@ -1161,7 +1184,7 @@ hidpp10_onboard_profiles_parse_macro(struct hidpp10_device *device,
 	hidpp_log_raw(&device->base, "*** macro starts at (0x%02x, 0x%04x) ***\n", page, offset);
 
 	rc = hidpp10_onboard_profiles_read_macro(device, page, offset, &macro);
-	if (rc < 0)
+	if (rc)
 		return rc;
 
 	count = rc;
@@ -1532,57 +1555,62 @@ hidpp10_read_profile(struct hidpp10_device *dev, uint8_t number)
 		}
 		profile->initialized = 1;
 
-		hidpp_log_buf_raw(&dev->base,
-				  "+++++++++++++++++++ Profile data: +++++++++++++++++ \n",
-				  data->data, 78);
+		hidpp_log_raw(&dev->base,
+			      "+++++++++++++++++++ Profile data: +++++++++++++++++ \n");
+		for (size_t x = 0; x < 78; x += 8) {
+			hidpp_log_buf_raw(&dev->base, NULL, &data->data[x], min(8, 78 - x));
+		}
+		hidpp_log_raw(&dev->base,
+			      "+++++++++++++++++++ Profile data end +++++++++++++++++ \n");
+
 	}
 
 	hidpp_log_raw(&dev->base, "Profile %d:\n", number);
 	for (i = 0; i < 5; i++) {
 		hidpp_log_raw(&dev->base,
-			"DPI mode: %dx%d dpi\n",
+			"  DPI mode: %dx%d dpi\n",
 			profile->dpi_modes[i].xres,
 			profile->dpi_modes[i].yres);
 		hidpp_log_raw(&dev->base,
-			"LED status: 1:%s 2:%s 3:%s 4:%s\n",
+			"  LED status: 1:%s 2:%s 3:%s 4:%s\n",
 			profile->dpi_modes[i].led[0] ? "on" : "off",
 			profile->dpi_modes[i].led[1] ? "on" : "off",
 			profile->dpi_modes[i].led[2] ? "on" : "off",
 			profile->dpi_modes[i].led[3] ? "on" : "off");
 	}
-	hidpp_log_raw(&dev->base, "Angle correction: %d\n", profile->angle_correction);
-	hidpp_log_raw(&dev->base, "Default DPI mode: %d\n", profile->default_dpi_mode);
-	hidpp_log_raw(&dev->base, "Refresh rate: %d\n", profile->refresh_rate);
+	hidpp_log_raw(&dev->base, "  Angle correction: %d\n", profile->angle_correction);
+	hidpp_log_raw(&dev->base, "  Default DPI mode: %d\n", profile->default_dpi_mode);
+	hidpp_log_raw(&dev->base, "  Refresh rate: %d\n", profile->refresh_rate);
 	for (i = 0; i < 13; i++) {
 		union hidpp10_button *button = &profile->buttons[i];
 		switch (button->any.type) {
 		case PROFILE_BUTTON_TYPE_BUTTON:
 			hidpp_log_raw(&dev->base,
-				"Button %zd: button %d\n",
+				"  Button %zd: button %d\n",
 				i,
 				button->button.button);
 			break;
 		case PROFILE_BUTTON_TYPE_KEYS:
 			hidpp_log_raw(&dev->base,
-				"Button %zd: key %d modifier %x\n",
+				"  Button %zd: key %d modifier %x\n",
 				i,
 				button->keys.key,
 				button->keys.modifier_flags);
 			break;
 		case PROFILE_BUTTON_TYPE_SPECIAL:
 			hidpp_log_raw(&dev->base,
-				"Button %zd: special %x\n",
+				"  Button %zd: special %x\n",
 				i,
 				button->special.special);
 			break;
 		case PROFILE_BUTTON_TYPE_CONSUMER_CONTROL:
 			hidpp_log_raw(&dev->base,
-				"Button %zd: consumer: %x\n",
+				"  Button %zd: consumer: %x\n",
 				i,
 				button->consumer_control.consumer_control);
 			break;
 		case PROFILE_BUTTON_TYPE_DISABLED:
-			hidpp_log_raw(&dev->base, "Button %zd: disabled\n", i);
+			hidpp_log_raw(&dev->base, "  Button %zd: disabled\n", i);
 			break;
 		default:
 			/* FIXME: this is the page number for the macro,
@@ -1757,7 +1785,7 @@ hidpp10_set_profile(struct hidpp10_device *dev, uint8_t number, struct hidpp10_p
 	}
 
 	crc = hidpp_crc_ccitt(page_data, HIDPP10_PAGE_SIZE - 2);
-	hidpp_set_unaligned_be_u16(&page_data[HIDPP10_PAGE_SIZE - 2], crc);
+	set_unaligned_be_u16(&page_data[HIDPP10_PAGE_SIZE - 2], crc);
 
 	/*
 	 * writing the data in several steps to prevent shroedinger state
@@ -1851,7 +1879,8 @@ hidpp10_get_led_status(struct hidpp10_device *dev,
 	uint8_t *status = led_status.msg.parameters;
 	int res;
 
-	hidpp_log_raw(&dev->base, "Fetching LED status\n");
+	hidpp_log_raw(&dev->base, "Fetching LED status (%#02x)\n",
+		      __CMD_LED_STATUS);
 
 	res = hidpp10_request_command(dev, &led_status);
 	if (res)
@@ -1877,7 +1906,8 @@ hidpp10_set_led_status(struct hidpp10_device *dev,
 	int res;
 	int i;
 
-	hidpp_log_raw(&dev->base, "Setting LED status\n");
+	hidpp_log_raw(&dev->base, "Setting LED status (%#02x)\n",
+		      __CMD_LED_STATUS);
 
 	for (i = 0; i < 6; i++) {
 		switch (led[i]) {
@@ -1928,7 +1958,8 @@ hidpp10_get_led_intensity(struct hidpp10_device *dev,
 	uint8_t *intensity = led_intensity.msg.parameters;
 	int res;
 
-	hidpp_log_raw(&dev->base, "Fetching LED intensity\n");
+	hidpp_log_raw(&dev->base, "Fetching LED intensity (%#02x)\n",
+		      __CMD_LED_INTENSITY);
 
 	res = hidpp10_request_command(dev, &led_intensity);
 	if (res)
@@ -1953,7 +1984,8 @@ hidpp10_set_led_intensity(struct hidpp10_device *dev,
 	uint8_t *intensity = led_intensity.msg.parameters;
 	int res;
 
-	hidpp_log_raw(&dev->base, "Setting LED intensity\n");
+	hidpp_log_raw(&dev->base, "Setting LED intensity (%#02x)\n",
+		      __CMD_LED_INTENSITY);
 
 	intensity[0]  = led_intensity_in_percent[0]/10 & 0xF;
 	intensity[0] |= (led_intensity_in_percent[1]/10 & 0xF) << 4;
@@ -1992,7 +2024,8 @@ hidpp10_get_led_color(struct hidpp10_device *dev,
 	union hidpp10_message led_color = CMD_LED_COLOR(idx, GET_REGISTER_REQ);
 	int res;
 
-	hidpp_log_raw(&dev->base, "Fetching LED color\n");
+	hidpp_log_raw(&dev->base, "Fetching LED color (%#02x)\n",
+		      __CMD_LED_COLOR);
 
 	res = hidpp10_request_command(dev, &led_color);
 	if (res)
@@ -2015,7 +2048,8 @@ hidpp10_set_led_color(struct hidpp10_device *dev,
 	union hidpp10_message led_color = CMD_LED_COLOR(idx, SET_REGISTER_REQ);
 	int res;
 
-	hidpp_log_raw(&dev->base, "Setting LED color\n");
+	hidpp_log_raw(&dev->base, "Setting LED color (%#02x)\n",
+		      __CMD_LED_COLOR);
 
 	led_color.msg.parameters[0] = red;
 	led_color.msg.parameters[1] = green;
@@ -2048,7 +2082,8 @@ hidpp10_get_optical_sensor_settings(struct hidpp10_device *dev,
 	union hidpp10_message sensor = CMD_OPTICAL_SENSOR_SETTINGS(idx, GET_REGISTER_REQ);
 	int res;
 
-	hidpp_log_raw(&dev->base, "Fetching optical sensor settings\n");
+	hidpp_log_raw(&dev->base, "Fetching optical sensor settings (%#02x)\n",
+		      __CMD_OPTICAL_SENSOR_SETTINGS);
 
 	res = hidpp10_request_command(dev, &sensor);
 	if (res)
@@ -2086,7 +2121,8 @@ hidpp10_get_current_resolution(struct hidpp10_device *dev,
 	union hidpp10_message resolution_long = CMD_CURRENT_RESOLUTION(REPORT_ID_SHORT, idx, GET_LONG_REGISTER_REQ);
 	int res;
 
-	hidpp_log_raw(&dev->base, "Fetching current resolution\n");
+	hidpp_log_raw(&dev->base, "Fetching current resolution (%#02x)\n",
+		      __CMD_CURRENT_RESOLUTION);
 
 	switch (dev->profile_type) {
 	case HIDPP10_PROFILE_G9:
@@ -2094,15 +2130,15 @@ hidpp10_get_current_resolution(struct hidpp10_device *dev,
 		if (res)
 			return res;
 		/* resolution is in 50dpi multiples */
-		*xres = *yres = hidpp10_get_dpi_value(dev, hidpp_get_unaligned_le_u16(&resolution.data[4]));
+		*xres = *yres = hidpp10_get_dpi_value(dev, get_unaligned_le_u16(&resolution.data[4]));
 		break;
         default:
 		res = hidpp10_request_command(dev, &resolution_long);
 		if (res)
 			return res;
 		/* resolution is in 50dpi multiples */
-		*xres = hidpp10_get_dpi_value(dev, hidpp_get_unaligned_le_u16(&resolution_long.data[4]));
-		*yres = hidpp10_get_dpi_value(dev, hidpp_get_unaligned_le_u16(&resolution_long.data[6]));
+		*xres = hidpp10_get_dpi_value(dev, get_unaligned_le_u16(&resolution_long.data[4]));
+		*yres = hidpp10_get_dpi_value(dev, get_unaligned_le_u16(&resolution_long.data[6]));
         }
 
 	return 0;
@@ -2118,14 +2154,17 @@ hidpp10_set_current_resolution(struct hidpp10_device *dev,
 	union hidpp10_message resolution_long = CMD_CURRENT_RESOLUTION(REPORT_ID_LONG, idx, SET_LONG_REGISTER_REQ);
 	int res;
 
+	hidpp_log_raw(&dev->base, "Setting current resolution (%#02x)\n",
+		      __CMD_CURRENT_RESOLUTION);
+
 	switch (dev->profile_type) {
 	case HIDPP10_PROFILE_G9:
 		resolution.data[4] = hidpp10_get_dpi_mapping(dev, xres);
 		res = hidpp10_request_command(dev, &resolution);
 		break;
         default:
-		hidpp_set_unaligned_le_u16(&resolution_long.data[4], hidpp10_get_dpi_mapping(dev, xres));
-		hidpp_set_unaligned_le_u16(&resolution_long.data[6], hidpp10_get_dpi_mapping(dev, yres));
+		set_unaligned_le_u16(&resolution_long.data[4], hidpp10_get_dpi_mapping(dev, xres));
+		set_unaligned_le_u16(&resolution_long.data[6], hidpp10_get_dpi_mapping(dev, yres));
 
 		res = hidpp10_request_command(dev, &resolution_long);
 		break;
@@ -2157,7 +2196,8 @@ hidpp10_get_usb_refresh_rate(struct hidpp10_device *dev,
 	union hidpp10_message refresh = CMD_USB_REFRESH_RATE(idx, GET_REGISTER_REQ);
 	int res;
 
-	hidpp_log_raw(&dev->base, "Fetching USB refresh rate\n");
+	hidpp_log_raw(&dev->base, "Fetching USB refresh rate (%#02x)\n",
+		      __CMD_USB_REFRESH_RATE);
 
 	res = hidpp10_request_command(dev, &refresh);
 	if (res)
@@ -2178,7 +2218,8 @@ hidpp10_set_usb_refresh_rate(struct hidpp10_device *dev,
 	unsigned idx = dev->index;
 	union hidpp10_message refresh = CMD_USB_REFRESH_RATE(idx, GET_REGISTER_REQ);
 
-	hidpp_log_raw(&dev->base, "Setting USB refresh rate\n");
+	hidpp_log_raw(&dev->base, "Setting USB refresh rate (%#02x)\n",
+		      __CMD_USB_REFRESH_RATE);
 
 	refresh.msg.parameters[0] = 1000/rate;
 
@@ -2483,7 +2524,7 @@ hidpp10_read_page(struct hidpp10_device *dev, uint8_t page,
 	}
 
 	crc = hidpp_crc_ccitt(bytes, HIDPP10_PAGE_SIZE - 2);
-	read_crc = hidpp_get_unaligned_be_u16(&bytes[HIDPP10_PAGE_SIZE - 2]);
+	read_crc = get_unaligned_be_u16(&bytes[HIDPP10_PAGE_SIZE - 2]);
 
 	if (crc != read_crc)
 		return -EILSEQ; /* return illegal sequence */
@@ -2573,7 +2614,7 @@ hidpp10_get_pairing_information(struct hidpp10_device *dev,
 		return -1;
 
 	*report_interval = pairing_information.msg.string[2];
-	*wpid = hidpp_get_unaligned_be_u16(&pairing_information.msg.string[3]);
+	*wpid = get_unaligned_be_u16(&pairing_information.msg.string[3]);
 	*device_type = pairing_information.msg.string[7];
 
 	return 0;
@@ -2613,7 +2654,7 @@ hidpp10_get_extended_pairing_information(struct hidpp10_device *dev,
 	if (res)
 		return -1;
 
-	*serial = hidpp_get_unaligned_be_u32(&info.msg.string[1]);
+	*serial = get_unaligned_be_u32(&info.msg.string[1]);
 
 	return 0;
 }
@@ -2665,7 +2706,7 @@ hidpp10_get_firmare_information(struct hidpp10_device *dev,
 	res = hidpp10_request_command(dev, &build_information);
 	if (res)
 		return res;
-	build = hidpp_get_unaligned_be_u16(&build_information.msg.string[1]);
+	build = get_unaligned_be_u16(&build_information.msg.string[1]);
 
 	*major_out = maj;
 	*minor_out = min;
