@@ -58,6 +58,7 @@ hidpp20_feature_get_name(uint16_t feature)
 	CASE_RETURN_STRING(HIDPP_PAGE_DEVICE_NAME);
 	CASE_RETURN_STRING(HIDPP_PAGE_RESET);
 	CASE_RETURN_STRING(HIDPP_PAGE_BATTERY_LEVEL_STATUS);
+	CASE_RETURN_STRING(HIDPP_PAGE_BATTERY_VOLTAGE);
 	CASE_RETURN_STRING(HIDPP_PAGE_KBD_REPROGRAMMABLE_KEYS);
 	CASE_RETURN_STRING(HIDPP_PAGE_SPECIAL_KEYS_BUTTONS);
 	CASE_RETURN_STRING(HIDPP_PAGE_WIRELESS_DEVICE_STATUS);
@@ -65,6 +66,7 @@ hidpp20_feature_get_name(uint16_t feature)
 	CASE_RETURN_STRING(HIDPP_PAGE_ADJUSTABLE_DPI);
 	CASE_RETURN_STRING(HIDPP_PAGE_ADJUSTABLE_REPORT_RATE);
 	CASE_RETURN_STRING(HIDPP_PAGE_COLOR_LED_EFFECTS);
+	CASE_RETURN_STRING(HIDPP_PAGE_RGB_EFFECTS);
 	CASE_RETURN_STRING(HIDPP_PAGE_ONBOARD_PROFILES);
 	CASE_RETURN_STRING(HIDPP_PAGE_MOUSE_BUTTON_SPY);
 	default:
@@ -244,7 +246,7 @@ hidpp_root_get_feature(struct hidpp20_device *device,
 		.msg.address = CMD_ROOT_GET_FEATURE,
 	};
 
-	hidpp_set_unaligned_be_u16(&msg.msg.parameters[0], feature);
+	set_unaligned_be_u16(&msg.msg.parameters[0], feature);
 
 	rc = hidpp20_request_command(device, &msg);
 	if (rc)
@@ -332,7 +334,7 @@ hidpp20_feature_set_get_feature_id(struct hidpp20_device *device,
 	if (rc)
 		return rc;
 
-	*feature = hidpp_get_unaligned_be_u16(msg.msg.parameters);
+	*feature = get_unaligned_be_u16(msg.msg.parameters);
 	*type = msg.msg.parameters[2];
 
 	return 0;
@@ -430,6 +432,41 @@ hidpp20_batterylevel_get_battery_level(struct hidpp20_device *device,
 	return msg.msg.parameters[2];
 }
 
+/* -------------------------------------------------------------------------- */
+/* 0x1001: Battery voltage                                                    */
+/* -------------------------------------------------------------------------- */
+
+#define CMD_BATTERY_VOLTAGE_GET_BATTERY_VOLTAGE 0x00
+#define CMD_BATTERY_VOLTAGE_GET_SHOW_BATTERY_STATUS 0x10
+
+int
+hidpp20_batteryvoltage_get_battery_voltage(struct hidpp20_device *device,
+					   uint16_t *voltage)
+{
+	uint8_t feature_index;
+	union hidpp20_message msg = {
+		.msg.report_id = REPORT_ID_SHORT,
+		.msg.device_idx = device->index,
+		.msg.address = CMD_BATTERY_VOLTAGE_GET_BATTERY_VOLTAGE,
+	};
+	int rc;
+
+	feature_index = hidpp_root_get_feature_idx(device,
+						   HIDPP_PAGE_BATTERY_VOLTAGE);
+	if (feature_index == 0)
+		return -ENOTSUP;
+
+	msg.msg.sub_id = feature_index;
+
+	rc = hidpp20_request_command(device, &msg);
+	if (rc)
+		return rc;
+
+	*voltage = get_unaligned_be_u16(msg.msg.parameters);
+
+	return msg.msg.parameters[2];
+}
+
 /* -------------------------------------------------------------------------- */
 /* 0x1300: Non-RGB led support                                                */
 /* -------------------------------------------------------------------------- */
@@ -685,10 +722,10 @@ int hidpp20_led_sw_control_set_led_state(struct hidpp20_device* device,
 		return -EINVAL;
 
 	msg.msg.parameters[0] = state->index;
-	hidpp_set_unaligned_be_u16(&msg.msg.parameters[1], state->mode);
-	hidpp_set_unaligned_be_u16(&msg.msg.parameters[3], state->blink.index);
-	hidpp_set_unaligned_be_u16(&msg.msg.parameters[5], state->blink.on_time);
-	hidpp_set_unaligned_be_u16(&msg.msg.parameters[7], state->blink.off_time);
+	set_unaligned_be_u16(&msg.msg.parameters[1], state->mode);
+	set_unaligned_be_u16(&msg.msg.parameters[3], state->blink.index);
+	set_unaligned_be_u16(&msg.msg.parameters[5], state->blink.on_time);
+	set_unaligned_be_u16(&msg.msg.parameters[7], state->blink.off_time);
 
 	rc = hidpp20_request_command(device, &msg);
 
@@ -742,8 +779,8 @@ hidpp20_kbd_reprogrammable_keys_get_info(struct hidpp20_device *device,
 	if (rc)
 		return rc;
 
-	control->control_id = hidpp_get_unaligned_be_u16(&msg.msg.parameters[0]);
-	control->task_id = hidpp_get_unaligned_be_u16(&msg.msg.parameters[2]);
+	control->control_id = get_unaligned_be_u16(&msg.msg.parameters[0]);
+	control->task_id = get_unaligned_be_u16(&msg.msg.parameters[2]);
 	control->flags = msg.msg.parameters[4];
 
 	return 0;
@@ -906,7 +943,7 @@ hidpp20_color_led_effects_get_zone_infos(struct hidpp20_device *device,
 			      "led_info %d: location: %d type %s num_effects: %d persistency_caps: 0x%02x\n",
 			      info->index,
 			      info->location,
-			      hidpp20_8070_get_location_mapping_name(info->location),
+			      hidpp20_led_get_location_mapping_name(info->location),
 			      info->num_effects,
 			      info->persistency_caps);
 	}
@@ -948,9 +985,179 @@ hidpp20_color_led_effect_get_zone_effect_info(struct hidpp20_device *device,
 	info->zone_index = msg.msg.parameters[0];
 	info->zone_effect_index = msg.msg.parameters[1];
 
-	info->effect_id = hidpp_get_unaligned_be_u16(&msg.msg.parameters[2]);
-	info->effect_caps = hidpp_get_unaligned_be_u16(&msg.msg.parameters[4]);
-	info->effect_period = hidpp_get_unaligned_be_u16(&msg.msg.parameters[6]);
+	info->effect_id = get_unaligned_be_u16(&msg.msg.parameters[2]);
+	info->effect_caps = get_unaligned_be_u16(&msg.msg.parameters[4]);
+	info->effect_period = get_unaligned_be_u16(&msg.msg.parameters[6]);
+
+	return 0;
+}
+
+/* -------------------------------------------------------------------------- */
+/* 0x8071: RGB Effects                                                        */
+/* -------------------------------------------------------------------------- */
+
+#define CMD_RGB_EFFECTS_GET_INFO				0x00
+#define CMD_RGB_EFFECTS_SET_RGB_CLUSTER_EFFECT			0x10
+#define CMD_RGB_EFFECTS_SET_MULTI_LED_RGB_CLUSTER_PATTERN	0x20
+#define CMD_RGB_EFFECTS_MANAGE_NV_CONFIG			0x30
+#define CMD_RGB_EFFECTS_MANAGE_RGB_LED_BIN_INFO			0x40
+#define CMD_RGB_EFFECTS_MANAGE_SW_CONTROL			0x50
+#define CMD_RGB_EFFECTS_SET_EFFECT_SYNC_CORRECTION		0x60
+#define CMD_RGB_EFFECTS_MANAGE_RGB_POWER_MODE_CONFIG		0x70
+#define CMD_RGB_EFFECTS_MANAGE_RGB_POWER_MODE			0x80
+
+int
+hidpp20_rgb_effects_get_device_info(struct hidpp20_device *device,
+				    struct hidpp20_rgb_device_info *info)
+{
+	int rc;
+	union hidpp20_message msg = {
+		.msg.report_id = REPORT_ID_LONG,
+		.msg.device_idx = device->index,
+		.msg.address = CMD_RGB_EFFECTS_GET_INFO,
+		.msg.parameters[0] = HIDPP20_RGB_EFFECTS_INDEX_ALL,
+		.msg.parameters[1] = HIDPP20_RGB_EFFECTS_INDEX_ALL,
+		.msg.parameters[2] = HIDPP20_RGB_EFFECTS_TOI_GENERAL,
+	};
+	uint8_t feature_index;
+
+	feature_index = hidpp_root_get_feature_idx(device,
+						   HIDPP_PAGE_RGB_EFFECTS);
+	if (feature_index == 0)
+		return -ENOTSUP;
+
+	msg.msg.sub_id = feature_index;
+
+	rc = hidpp20_request_command(device, &msg);
+	if (rc)
+		return rc;
+
+	info->cluster_index = msg.msg.parameters[0];
+	info->effect_index = msg.msg.parameters[1];
+	info->cluster_count = msg.msg.parameters[2];
+
+	info->nv_caps = get_unaligned_be_u16(&msg.msg.parameters[3]);
+	info->ext_caps = get_unaligned_be_u16(&msg.msg.parameters[4]);
+
+	return 0;
+}
+
+int
+hidpp20_rgb_effects_get_cluster_info(struct hidpp20_device *device,
+				     uint8_t cluster_index,
+				     struct hidpp20_rgb_cluster_info *info)
+{
+	int rc;
+	union hidpp20_message msg = {
+		.msg.report_id = REPORT_ID_LONG,
+		.msg.device_idx = device->index,
+		.msg.address = CMD_RGB_EFFECTS_GET_INFO,
+		.msg.parameters[0] = cluster_index,
+		.msg.parameters[1] = HIDPP20_RGB_EFFECTS_INDEX_ALL,
+		.msg.parameters[2] = HIDPP20_RGB_EFFECTS_TOI_GENERAL,
+	};
+	uint8_t feature_index;
+
+	feature_index = hidpp_root_get_feature_idx(device,
+						   HIDPP_PAGE_RGB_EFFECTS);
+	if (feature_index == 0)
+		return -ENOTSUP;
+
+	msg.msg.sub_id = feature_index;
+
+	rc = hidpp20_request_command(device, &msg);
+	if (rc)
+		return rc;
+
+	info->index = msg.msg.parameters[0];
+	info->effect_index = msg.msg.parameters[1];
+
+	info->location = get_unaligned_be_u16(&msg.msg.parameters[2]);
+	info->num_effects = get_unaligned_be_u16(&msg.msg.parameters[4]);
+	info->persistency_caps = get_unaligned_be_u16(&msg.msg.parameters[6]);
+
+	return 0;
+}
+
+int
+hidpp20_rgb_effects_get_cluster_infos(struct hidpp20_device *device,
+				      struct hidpp20_rgb_cluster_info **infos_list)
+{
+	struct hidpp20_rgb_cluster_info *i_list, *info;
+	struct hidpp20_rgb_device_info device_info = {0};
+	uint8_t num_infos;
+	unsigned i;
+	int rc;
+
+	rc = hidpp20_rgb_effects_get_device_info(device, &device_info);
+	if (rc < 0)
+		return rc;
+
+	num_infos = device_info.cluster_count;
+	if (num_infos == 0) {
+		*infos_list = NULL;
+		return 0;
+	}
+
+	i_list = zalloc(num_infos * sizeof(struct hidpp20_rgb_cluster_info));
+
+	for (i = 0; i < num_infos; i++) {
+		info = &i_list[i];
+		info->index = i;
+		rc = hidpp20_rgb_effects_get_cluster_info(device, i, info);
+		if (rc)
+			goto err;
+
+		hidpp_log_raw(&device->base,
+			      "cluster_info %d: location: %d type %s num_effects: %d persistency_caps: 0x%02x\n",
+			      info->index,
+			      info->location,
+			      hidpp20_led_get_location_mapping_name(info->location),
+			      info->num_effects,
+			      info->persistency_caps);
+	}
+
+	*infos_list = i_list;
+	return num_infos;
+err:
+	free(i_list);
+	return rc;
+}
+
+int
+hidpp20_rgb_effects_get_effect_info(struct hidpp20_device *device,
+				    uint8_t cluster_index,
+				    uint8_t effect_index,
+				    struct hidpp20_rgb_effect_info *info)
+{
+	int rc;
+	union hidpp20_message msg = {
+		.msg.report_id = REPORT_ID_LONG,
+		.msg.device_idx = device->index,
+		.msg.address = CMD_RGB_EFFECTS_GET_INFO,
+		.msg.parameters[0] = cluster_index,
+		.msg.parameters[1] = effect_index,
+		.msg.parameters[2] = HIDPP20_RGB_EFFECTS_TOI_GENERAL,
+	};
+	uint8_t feature_index;
+
+	feature_index = hidpp_root_get_feature_idx(device,
+						   HIDPP_PAGE_RGB_EFFECTS);
+	if (feature_index == 0)
+		return -ENOTSUP;
+
+	msg.msg.sub_id = feature_index;
+
+	rc = hidpp20_request_command(device, &msg);
+	if (rc)
+		return rc;
+
+	info->cluster_index = msg.msg.parameters[0];
+	info->effect_index = msg.msg.parameters[1];
+
+	info->effect_id = get_unaligned_be_u16(&msg.msg.parameters[2]);
+	info->capabilities = get_unaligned_be_u16(&msg.msg.parameters[4]);
+	info->effect_period = get_unaligned_be_u16(&msg.msg.parameters[6]);
 
 	return 0;
 }
@@ -1000,8 +1207,8 @@ hidpp20_special_keys_buttons_get_info(struct hidpp20_device *device,
 	if (rc)
 		return rc;
 
-	control->control_id = hidpp_get_unaligned_be_u16(&msg.msg.parameters[0]);
-	control->task_id = hidpp_get_unaligned_be_u16(&msg.msg.parameters[2]);
+	control->control_id = get_unaligned_be_u16(&msg.msg.parameters[0]);
+	control->task_id = get_unaligned_be_u16(&msg.msg.parameters[2]);
 	control->flags = msg.msg.parameters[4];
 	control->position = msg.msg.parameters[5];
 	control->group = msg.msg.parameters[6];
@@ -1025,13 +1232,13 @@ hidpp20_special_keys_buttons_get_reporting(struct hidpp20_device *device,
 		.msg.address = CMD_SPECIAL_KEYS_BUTTONS_GET_REPORTING,
 	};
 
-	hidpp_set_unaligned_be_u16(&msg.msg.parameters[0], control->control_id);
+	set_unaligned_be_u16(&msg.msg.parameters[0], control->control_id);
 
 	rc = hidpp20_request_command(device, &msg);
 	if (rc)
 		return rc;
 
-	control->reporting.remapped = hidpp_get_unaligned_be_u16(&msg.msg.parameters[3]);
+	control->reporting.remapped = get_unaligned_be_u16(&msg.msg.parameters[3]);
 	control->reporting.raw_XY = !!(msg.msg.parameters[2] & 0x10);
 	control->reporting.persist = !!(msg.msg.parameters[2] & 0x04);
 	control->reporting.divert = !!(msg.msg.parameters[2] & 0x01);
@@ -1125,8 +1332,8 @@ hidpp20_special_key_mouse_set_control(struct hidpp20_device *device,
 		.msg.address = CMD_SPECIAL_KEYS_BUTTONS_SET_REPORTING,
 	};
 
-	hidpp_set_unaligned_be_u16(&msg.msg.parameters[0], control->control_id);
-	hidpp_set_unaligned_be_u16(&msg.msg.parameters[3], control->reporting.remapped);
+	set_unaligned_be_u16(&msg.msg.parameters[0], control->control_id);
+	set_unaligned_be_u16(&msg.msg.parameters[3], control->reporting.remapped);
 
 	feature_index = hidpp_root_get_feature_idx(device,
 						   HIDPP_PAGE_SPECIAL_KEYS_BUTTONS);
@@ -1177,7 +1384,7 @@ hidpp20_mousepointer_get_mousepointer_info(struct hidpp20_device *device,
 	if (rc)
 		return rc;
 
-	*resolution = hidpp_get_unaligned_be_u16(msg.msg.parameters);
+	*resolution = get_unaligned_be_u16(msg.msg.parameters);
 	*flags = msg.msg.parameters[2];
 
 	return 0;
@@ -1233,8 +1440,8 @@ hidpp20_adjustable_dpi_get_dpi_list(struct hidpp20_device *device,
 
 	sensor->index = msg.msg.parameters[0];
 	while (i < LONG_MESSAGE_LENGTH - 4U &&
-	       hidpp_get_unaligned_be_u16(&msg.msg.parameters[i]) != 0) {
-		uint16_t value = hidpp_get_unaligned_be_u16(&msg.msg.parameters[i]);
+	       get_unaligned_be_u16(&msg.msg.parameters[i]) != 0) {
+		uint16_t value = get_unaligned_be_u16(&msg.msg.parameters[i]);
 
 		if (value > 0xe000) {
 			sensor->dpi_steps = value - 0xe000;
@@ -1269,8 +1476,8 @@ hidpp20_adjustable_dpi_get_dpi(struct hidpp20_device *device,
 	if (rc)
 		return rc;
 
-	sensor->dpi = hidpp_get_unaligned_be_u16(&msg.msg.parameters[1]);
-	sensor->default_dpi = hidpp_get_unaligned_be_u16(&msg.msg.parameters[3]);
+	sensor->dpi = get_unaligned_be_u16(&msg.msg.parameters[1]);
+	sensor->default_dpi = get_unaligned_be_u16(&msg.msg.parameters[3]);
 
 	return 0;
 }
@@ -1344,7 +1551,7 @@ int hidpp20_adjustable_dpi_set_sensor_dpi(struct hidpp20_device *device,
 		.msg.parameters[0] = sensor->index,
 	};
 
-	hidpp_set_unaligned_be_u16(&msg.msg.parameters[1], dpi);
+	set_unaligned_be_u16(&msg.msg.parameters[1], dpi);
 
 	feature_index = hidpp_root_get_feature_idx(device,
 						   HIDPP_PAGE_ADJUSTABLE_DPI);
@@ -1357,7 +1564,7 @@ int hidpp20_adjustable_dpi_set_sensor_dpi(struct hidpp20_device *device,
 	if (rc)
 		return rc;
 
-	returned_parameters = hidpp_get_unaligned_be_u16(&msg.msg.parameters[1]);
+	returned_parameters = get_unaligned_be_u16(&msg.msg.parameters[1]);
 
 	/* version 0 of the protocol does not echo the parameters */
 	if (returned_parameters != dpi && returned_parameters)
@@ -1427,8 +1634,15 @@ int hidpp20_adjustable_report_rate_get_report_rate_list(struct hidpp20_device *d
 #define HIDPP20_ONBOARD_PROFILES_PROFILE_TYPE_G402	0x01
 #define HIDPP20_ONBOARD_PROFILES_PROFILE_TYPE_G303	0x02
 #define HIDPP20_ONBOARD_PROFILES_PROFILE_TYPE_G900	0x03
+#define HIDPP20_ONBOARD_PROFILES_PROFILE_TYPE_G903_HERO	0x04
 #define HIDPP20_ONBOARD_PROFILES_MACRO_TYPE_G402	0x01
 
+#define HIDPP20_USER_PROFILES_G402			0x0000
+#define HIDPP20_ROM_PROFILES_G402			0x0100
+
+#define HIDPP20_PROFILE_DIR_END				0xFFFF
+#define HIDPP20_PROFILE_DIR_ENABLED			2
+
 union hidpp20_internal_profile {
 	uint8_t data[HIDPP20_PROFILE_SIZE];
 	struct {
@@ -1439,7 +1653,9 @@ union hidpp20_internal_profile {
 		struct hidpp20_color profile_color;
 		uint8_t power_mode;
 		uint8_t angle_snapping;
-		uint8_t reserved[14];
+		uint8_t reserved[10];
+		uint16_t powersave_timeout;
+		uint16_t poweroff_timeout;
 		union hidpp20_button_binding buttons[16];
 		union hidpp20_button_binding alternate_buttons[16];
 		union {
@@ -1447,7 +1663,8 @@ union hidpp20_internal_profile {
 			uint8_t raw[16 * 3];
 		} name;
 		struct hidpp20_internal_led leds[2]; /* G303, g502, g900 only */
-		uint8_t free[24];
+		struct hidpp20_internal_led alt_leds[2];
+		uint8_t free[2];
 		uint16_t crc;
 	} __attribute__((packed)) profile;
 };
@@ -1499,13 +1716,15 @@ hidpp20_onboard_profiles_read_sector(struct hidpp20_device *device,
 		.msg.address = CMD_ONBOARD_PROFILES_MEMORY_READ,
 	};
 
+	hidpp_log_debug(&device->base, "Reading sector 0x%04x\n", sector);
+
 	feature_index = hidpp_root_get_feature_idx(device,
 						   HIDPP_PAGE_ONBOARD_PROFILES);
 	if (feature_index == 0)
 		return -ENOTSUP;
 
 	msg.msg.sub_id = feature_index;
-	hidpp_set_unaligned_be_u16(&msg.msg.parameters[0], sector);
+	set_unaligned_be_u16(&msg.msg.parameters[0], sector);
 
 	count = sector_size;
 
@@ -1516,14 +1735,14 @@ hidpp20_onboard_profiles_read_sector(struct hidpp20_device *device,
 		 * less than 16 bytes to read we need to read from sector_size - 16
 		 */
 		offset = (sector_size - offset < 16) ? sector_size - 16 : offset;
-		hidpp_set_unaligned_be_u16(&msg.msg.parameters[2], offset);
+		set_unaligned_be_u16(&msg.msg.parameters[2], offset);
 		buf = msg;
 		rc = hidpp20_request_command(device, &buf);
 		if (rc)
 			return rc;
 
 		/* msg.msg.parameters is guaranteed to have a size >= 16 */
-		memcpy(data + offset, buf.msg.parameters, min(count, 16));
+		memcpy(data + offset, buf.msg.parameters, 16);
 
 		/*
 		 * no need to check for count >= 0:
@@ -1544,7 +1763,10 @@ hidpp20_onboard_profiles_is_sector_valid(struct hidpp20_device *device,
 	uint16_t crc, read_crc;
 
 	crc = hidpp_crc_ccitt(data, sector_size - 2);
-	read_crc = hidpp_get_unaligned_be_u16(&data[sector_size - 2]);
+	read_crc = get_unaligned_be_u16(&data[sector_size - 2]);
+
+	if (crc != read_crc)
+		hidpp_log_debug(&device->base, "Invalid CRC (%04x != %04x)\n", read_crc, crc);
 
 	return crc == read_crc;
 }
@@ -1564,9 +1786,9 @@ hidpp20_onboard_profiles_write_start(struct hidpp20_device *device,
 		.msg.address = CMD_ONBOARD_PROFILES_MEMORY_ADDR_WRITE,
 	};
 
-	hidpp_set_unaligned_be_u16(&msg.msg.parameters[0], sector);
-	hidpp_set_unaligned_be_u16(&msg.msg.parameters[2], sub_address);
-	hidpp_set_unaligned_be_u16(&msg.msg.parameters[4], count);
+	set_unaligned_be_u16(&msg.msg.parameters[0], sector);
+	set_unaligned_be_u16(&msg.msg.parameters[2], sub_address);
+	set_unaligned_be_u16(&msg.msg.parameters[4], count);
 
 	rc = hidpp20_request_command(device, &msg);
 	if (rc)
@@ -1634,7 +1856,7 @@ hidpp20_onboard_profiles_write_sector(struct hidpp20_device *device,
 
 	if (write_crc) {
 		crc = hidpp_crc_ccitt(data, sector_size - 2);
-		hidpp_set_unaligned_be_u16(&data[sector_size - 2], crc);
+		set_unaligned_be_u16(&data[sector_size - 2], crc);
 	}
 
 	rc = hidpp20_onboard_profiles_write_start(device,
@@ -1831,7 +2053,8 @@ hidpp20_onboard_profiles_validate(struct hidpp20_device *device,
 
 	if ((info->profile_format_id != HIDPP20_ONBOARD_PROFILES_PROFILE_TYPE_G402) &&
 	    (info->profile_format_id != HIDPP20_ONBOARD_PROFILES_PROFILE_TYPE_G303) &&
-	    (info->profile_format_id != HIDPP20_ONBOARD_PROFILES_PROFILE_TYPE_G900)) {
+	    (info->profile_format_id != HIDPP20_ONBOARD_PROFILES_PROFILE_TYPE_G900) &&
+	    (info->profile_format_id != HIDPP20_ONBOARD_PROFILES_PROFILE_TYPE_G903_HERO)) {
 		hidpp_log_error(&device->base,
 				"Profile layout not supported: 0x%02x.\n",
 				info->profile_format_id);
@@ -2325,12 +2548,18 @@ hidpp20_onboard_profiles_parse_profile(struct hidpp20_device *device,
 	profile->default_dpi = pdata->profile.default_dpi;
 	profile->switched_dpi = pdata->profile.switched_dpi;
 
+	profile->powersave_timeout = pdata->profile.powersave_timeout;
+	profile->poweroff_timeout = pdata->profile.poweroff_timeout;
+
 	for (i = 0; i < 5; i++) {
-		profile->dpi[i] = hidpp_get_unaligned_le_u16(&data[2 * i + 3]);
+		profile->dpi[i] = get_unaligned_le_u16(&data[2 * i + 3]);
 	}
 
 	for (i = 0; i < profiles_list->num_leds; i++)
+	{
 		hidpp20_onboard_profiles_read_led(&profile->leds[i], pdata->profile.leds[i]);
+		hidpp20_onboard_profiles_read_led(&profile->alt_leds[i], pdata->profile.alt_leds[i]);
+	}
 
 	hidpp20_buttons_to_cpu(device, profiles_list, profile, pdata->profile.buttons, profiles_list->num_buttons);
 
@@ -2349,16 +2578,18 @@ hidpp20_onboard_profiles_parse_profile(struct hidpp20_device *device,
 	return 0;
 }
 
-static int
-hidpp20_onboard_profiles_parse(struct hidpp20_device *device,
-			      struct hidpp20_profiles *profiles,
-			      uint16_t dict_address,
-			      bool check_crc)
+int
+hidpp20_onboard_profiles_initialize(struct hidpp20_device *device,
+				    struct hidpp20_profiles *profiles)
 {
 	_cleanup_free_ uint8_t *data = NULL;
 	int rc;
 	unsigned i;
+	uint16_t addr;
 	bool crc_valid;
+	bool read_userdata = true;
+
+	assert(profiles);
 
 	for (i = 0; i < profiles->num_profiles; i++) {
 		profiles->profiles[i].address = 0;
@@ -2368,7 +2599,7 @@ hidpp20_onboard_profiles_parse(struct hidpp20_device *device,
 	data = hidpp20_onboard_profiles_allocate_sector(profiles);
 
 	rc = hidpp20_onboard_profiles_read_sector(device,
-						  dict_address,
+						  HIDPP20_USER_PROFILES_G402,
 						  profiles->sector_size,
 						  data);
 	if (rc < 0)
@@ -2377,92 +2608,64 @@ hidpp20_onboard_profiles_parse(struct hidpp20_device *device,
 	crc_valid = hidpp20_onboard_profiles_is_sector_valid(device,
 							     profiles->sector_size,
 							     data);
-	if (crc_valid || !check_crc) {
+	if (crc_valid) {
 		for (i = 0; i < profiles->num_profiles; i++) {
 			uint8_t *d = data + 4 * i;
 
-			if (d[0] == 0xFF && d[1] == 0xFF)
+			addr = get_unaligned_be_u16(d);
+
+			if(addr == HIDPP20_PROFILE_DIR_END)
 				break;
 
-			profiles->profiles[i].address = hidpp_get_unaligned_be_u16(d);
-			if (profiles->profiles[i].address != (dict_address | (i + 1)))
+			profiles->profiles[i].address = addr;
+
+			/* profile address sanity check */
+			if (profiles->profiles[i].address != (HIDPP20_USER_PROFILES_G402 | (i + 1)))
 				hidpp_log_info(&device->base,
-						"profile %d: error in the address: 0x%04x instead of 0x%04x\n",
-						i + 1,
-						profiles->profiles[i].address,
-						dict_address | (i + 1));
-			profiles->profiles[i].enabled = !!d[2];
+					       "profile %d: error in the address: 0x%04x instead of 0x%04x\n",
+					       i + 1,
+					       profiles->profiles[i].address,
+					       HIDPP20_USER_PROFILES_G402 | (i + 1));
+
+			profiles->profiles[i].enabled = !!d[HIDPP20_PROFILE_DIR_ENABLED];
 		}
+	} else {
+		hidpp_log_debug(&device->base, "Profile directory has an invalid CRC... Reading ROM profiles.\n");
+
+		read_userdata = false;
 	}
 
-	/*
-	 * Even if the crc is wrong, we still fetch all the profiles, so as
-	 * to have the current stored values if some profiles are legitimate.
-	 */
-	 for (i = 0; i < profiles->num_profiles; i++) {
-		if (profiles->profiles[i].address == 0x0000) {
-			switch (dict_address) {
-			case 0x0000:
-				profiles->profiles[i].address = i + 1;
-				break;
-			case 0x0100:
-				if (i >= profiles->num_rom_profiles) {
-					profiles->profiles[i].address = dict_address | profiles->num_rom_profiles;
-				} else {
-					profiles->profiles[i].address = dict_address | (i + 1);
-				}
-				break;
-			default:
-				hidpp_log_error(&device->base,
-						"this should never happen: dict address is 0x%04x\n",
-						dict_address);
-				return -EINVAL;
-			}
+	for (i = 0; i < profiles->num_profiles; i++) {
+		if (read_userdata) {
+			hidpp_log_debug(&device->base, "Parsing profile %u\n", i);
+			rc = hidpp20_onboard_profiles_parse_profile(device,
+								    profiles,
+								    i,
+								    true);
+
+			/* on fail to read the user profile fallback to the default profile */
+			if (rc)
+				hidpp_log_debug(&device->base, "Profile %u is bad. Falling back to the ROM settings.\n", i);
+			else
+				continue;
 		}
+
+		/* the number of rom profiles can be different than the number of user profiles
+		   so we if there are not enough rom profiles to populate all the user profiles
+		   we just use the first rom profile */
+		if (i + 1 > profiles->num_rom_profiles)
+			profiles->profiles[i].address = HIDPP20_ROM_PROFILES_G402 + 1;
+		else
+			profiles->profiles[i].address = HIDPP20_ROM_PROFILES_G402 + i + 1;
+
 		rc = hidpp20_onboard_profiles_parse_profile(device,
-							    profiles,
-							    i,
-							    check_crc);
-		if (rc == -EAGAIN)
-			crc_valid = false;
+							profiles,
+							i,
+							false);
+		if (rc < 0)
+			return rc;
 	}
 
-	if (check_crc && !crc_valid)
-		return -EAGAIN;
-
-	return 0;
-}
-
-int
-hidpp20_onboard_profiles_initialize(struct hidpp20_device *device,
-				    struct hidpp20_profiles *profiles)
-{
-	int rc;
-
-	assert(profiles);
-
-	rc = hidpp20_onboard_profiles_parse(device, profiles, 0x0000, true);
-	if (rc == 0)
-		return profiles->num_profiles;
-
-	if (rc != -EAGAIN) {
-		/* something went wrong */
-		return rc;
-	}
-
-	/*
-	 * One of the used sector has a bad crc, the device is using the ROM
-	 * profiles.
-	 */
-
-	hidpp_log_debug(&device->base, "device is using ROM settings\n");
-
-	rc = hidpp20_onboard_profiles_parse(device, profiles, 0x0100, false);
-	if (rc < 0)
-		return rc;
-
-	hidpp_log_debug(&device->base, "ROM settings loaded\n");
-
 	return profiles->num_profiles;
 }
 
@@ -2531,12 +2734,19 @@ hidpp20_onboard_profiles_write_profile(struct hidpp20_device *device,
 	pdata->profile.default_dpi = profile->default_dpi;
 	pdata->profile.switched_dpi = profile->switched_dpi;
 
+	pdata->profile.powersave_timeout = profile->powersave_timeout;
+	pdata->profile.poweroff_timeout = profile->poweroff_timeout;
+
 	for (i = 0; i < 5; i++) {
 		pdata->profile.dpi[i] = hidpp_cpu_to_le_u16(profile->dpi[i]);
 	}
 
 	for (i = 0; i < profiles_list->num_leds; i++)
+	{
 		hidpp20_onboard_profiles_write_led(&pdata->profile.leds[i], &profile->leds[i]);
+		/* we write the current led instead of the stored value */
+		hidpp20_onboard_profiles_write_led(&pdata->profile.alt_leds[i], &profile->leds[i]);
+	}
 
 	hidpp20_buttons_from_cpu(profile, pdata->profile.buttons, profiles_list->num_buttons);
 
