@@ -32,6 +32,10 @@
 #include "libratbag-util.h"
 #include "libratbag-hidraw.h"
 
+#ifdef NDEBUG
+#error "libratbag relies on assert(). Do not define NDEBUG"
+#endif
+
 #define RATBAG_LED_TYPE_UNKNOWN 0
 
 void
@@ -567,7 +571,7 @@ ratbag_resolution_set_cap(struct ratbag_resolution *res,
 {
 	assert(cap <= RATBAG_RESOLUTION_CAP_SEPARATE_XY_RESOLUTION);
 
-	res->capabilities = (1 << cap);
+	res->capabilities |= (1 << cap);
 }
 
 static inline void
