@@ -395,10 +395,12 @@ ratbag_assign_driver(struct ratbag_device *device,
 	if (!test_device) {
 		driver_name = ratbag_device_data_get_driver(device->data);
 	} else {
+		log_debug(device->ratbag, "This is a test device\n");
 		driver_name = "test_driver";
 	}
 
 	if (driver_name) {
+		log_debug(device->ratbag, "device assigned driver %s\n", driver_name);
 		rc = ratbag_test_driver(device, dev_id, driver_name, test_device);
 	} else {
 		rc = ratbag_driver_fallback_logitech(device, dev_id);
@@ -456,6 +458,8 @@ ratbag_device_new_from_udev_device(struct ratbag *ratbag,
 	if ((name = get_device_name(udev_device)) == 0)
 		goto out_err;
 
+	log_debug(ratbag, "New device: %s\n", name);
+
 	device = ratbag_device_new(ratbag, udev_device, name, &id);
 	if (!device || !device->data)
 		goto out_err;
@@ -624,7 +628,7 @@ ratbag_create_button(struct ratbag_profile *profile, unsigned int index)
 	button->index = index;
 	button->type = RATBAG_BUTTON_TYPE_UNKNOWN;
 
-	list_insert(&profile->buttons, &button->link);
+	list_append(&profile->buttons, &button->link);
 
 	return button;
 }
@@ -644,7 +648,7 @@ ratbag_create_led(struct ratbag_profile *profile, unsigned int index)
 	if (device->data)
 		led->type = ratbag_device_data_get_led_type(device->data, led->index);
 
-	list_insert(&profile->leds, &led->link);
+	list_append(&profile->leds, &led->link);
 
 	return led;
 }
@@ -669,7 +673,7 @@ ratbag_create_resolution(struct ratbag_profile *profile, int index)
 	res->profile = profile;
 	res->index = index;
 
-	list_insert(&profile->resolutions, &res->link);
+	list_append(&profile->resolutions, &res->link);
 
 	profile->num_resolutions++;
 }
@@ -693,7 +697,7 @@ ratbag_create_profile(struct ratbag_device *device,
 	profile->is_enabled = true;
 	profile->name = NULL;
 
-	list_insert(&device->profiles, &profile->link);
+	list_append(&device->profiles, &profile->link);
 	list_init(&profile->buttons);
 	list_init(&profile->leds);
 	list_init(&profile->resolutions);
