@@ -25,7 +25,6 @@
 
 import imp
 import os
-import shutil
 import subprocess
 import sys
 
@@ -36,9 +35,6 @@ RATBAGCTL_NAME = 'ratbagctl'
 RATBAGCTL_PATH = os.path.join('@MESON_BUILD_ROOT@', RATBAGCTL_NAME)
 RATBAGCTL_DEVEL_NAME = 'ratbagctl.devel'
 RATBAGCTL_DEVEL_PATH = os.path.join('@MESON_BUILD_ROOT@', RATBAGCTL_DEVEL_NAME)
-DBUS_CONF_DIR = '/etc/dbus-1/system.d'
-DBUS_CONF_NAME = 'org.freedesktop.ratbag_devel1.conf'
-DBUS_CONF_PATH = os.path.join(DBUS_CONF_DIR, DBUS_CONF_NAME)
 
 
 def import_non_standard_path(name, path):
@@ -61,12 +57,6 @@ def start_ratbagd(verbosity=0):
     from gi.repository import Gio
     import time
 
-    os.makedirs(DBUS_CONF_DIR, exist_ok=True)
-
-    # first copy the policy for the ratbagd daemon to be allowed to run
-    shutil.copy(os.path.join('@MESON_BUILD_ROOT@', DBUS_CONF_NAME),
-                DBUS_CONF_PATH)
-
     # FIXME: kill any running ratbagd.devel
 
     args = [os.path.join('@MESON_BUILD_ROOT@', "ratbagd.devel")]
@@ -88,15 +78,15 @@ def start_ratbagd(verbosity=0):
         proxy = Gio.DBusProxy.new_sync(dbus,
                                        Gio.DBusProxyFlags.NONE,
                                        None,
-                                       "org.freedesktop.ratbag_devel1_@ratbagd_sha@",
-                                       "/org/freedesktop/ratbag_devel1_@ratbagd_sha@",
-                                       "org.freedesktop.ratbag_devel1_@ratbagd_sha@.Manager",
+                                       "org.freedesktop.ratbag_devel1",
+                                       "/org/freedesktop/ratbag_devel1",
+                                       "org.freedesktop.ratbag_devel1.Manager",
                                        None)
         name_owner = proxy.get_name_owner()
         if name_owner is None:
             time.sleep(0.2)
 
-    os.environ['RATBAGCTL_DEVEL'] = "org.freedesktop.ratbag_devel1_@ratbagd_sha@"
+    os.environ['RATBAG_TEST'] = "1"
 
     if name_owner is None or ratbagd_process.poll() is not None:
         return None
@@ -111,10 +101,6 @@ def terminate_ratbagd(ratbagd):
             ratbagd.wait(5)
         except subprocess.TimeoutExpired:
             ratbagd.kill()
-    try:
-        os.unlink(DBUS_CONF_PATH)
-    except FileNotFoundError:
-        pass
 
 
 def sync_dbus():
@@ -125,14 +111,11 @@ def sync_dbus():
 
 ratbagctl = import_non_standard_path(RATBAGCTL_NAME, RATBAGCTL_PATH)
 
-from ratbagctl import open_ratbagd, get_parser, RatbagError, RatbagErrorCapability
+from ratbagctl import open_ratbagd, get_parser, RatbagError, RatbagErrorCapability  # NOQA
 
 __all__ = [
     RATBAGCTL_NAME,
     RATBAGCTL_PATH,
-    DBUS_CONF_DIR,
-    DBUS_CONF_NAME,
-    DBUS_CONF_PATH,
     start_ratbagd,
     terminate_ratbagd,
     open_ratbagd,
